program Example;

uses
  Windows, SysUtils, Classes, GDIPAPI, DWidgetGraphics, DWidget;

{$R *.res}

type
  TWidgetForm = class
  public
    procedure OnClick(Sender: TObject);
    procedure OnMouseEnter(Sender: TObject);
    procedure OnMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure OnRender(Sender: TObject; const Render: String;
      var DestRect, SrcRect: TRect; var Result: Boolean);
    procedure OnAfterRender(Sender: TObject);
    procedure OnAfterRenderClock(Sender: TObject);
    procedure OnTimer(Sender: TObject; const Timer: String);
    procedure OnClockMouseEnter(Sender: TObject);
    procedure OnMouseLeave(Sender: TObject);
  end;
  
var
  WidgetForm: TWidgetForm;
  WPanel, WButton, WLabel, WClock, WClose: TWidgetWindow;

procedure TWidgetForm.OnMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Button = mbLeft) then
  begin
    WButton.Render;
    WButton.Update;
  end;
end;

procedure TWidgetForm.OnMouseEnter(Sender: TObject);
begin
  WButton.Render;
  WButton.Update;
end;

procedure TWidgetForm.OnRender(Sender: TObject; const Render: String;
  var DestRect, SrcRect: TRect; var Result: Boolean);
begin
  if (wsLButtonDown in WButton.WidgetState) then
  begin
    Result := SameText(Render, 'press');
  end else
  if (wsMouseOn in WButton.WidgetState) then
  begin
    Result := SameText(Render, 'over');
  end else
  begin
    Result := SameText(Render, 'normal');
  end;
end;

procedure TWidgetForm.OnTimer(Sender: TObject; const Timer: String);
begin
  if SameText(Timer, 'timer') then
  begin
    TWidgetWindow(Sender).Render;
    TWidgetWindow(Sender).Update;
  end;
end;

procedure TWidgetForm.OnClockMouseEnter(Sender: TObject);
begin
  WClose.WidgetVisible := True;
end;

procedure TWidgetForm.OnMouseLeave(Sender: TObject);
begin
  WClose.WidgetVisible := False;
end;

procedure TWidgetForm.OnAfterRenderClock(Sender: TObject);
var
  Bmp: TWidgetBitmap;
  Matrix: GpMatrix;
begin
  Bmp := TWidgetWindow(Sender).GetRender('arrow');

  GdipCreateMatrix(Matrix);
  GdipRotateMatrix(Matrix, 30, MatrixOrderPrepend);

  GdipSetWorldTransform(TWidgetWindow(Sender).WidgetDIB.Context, Matrix);

  with TWidgetWindow(Sender).WidgetDIB do
    DrawBitmap(Classes.Rect(82, -28, 82 + Bmp.Width, -28 + Bmp.Height), Bmp.Bitmap, Bmp.Rect);


  GdipDeleteMatrix(Matrix);
end;

procedure TWidgetForm.OnAfterRender(Sender: TObject);
var
  Rect: TGpRectF;
  FontFamily: GpFontFamily;
  Font: GpFont;
  StringFormat: GpStringFormat;
  Brush: GpBrush;
  Text: String;
begin
  Text := FormatDateTime('hh:mm:ss', Now);

  GdipCreateFontFamilyFromName('Arial', NIL, FontFamily);
  GdipCreateFont(FontFamily, 10, FontStyleBold, Integer(UnitPoint), Font);
  GdipCreateStringFormat(0, LANG_NEUTRAL, StringFormat);
  GdipSetStringFormatAlign(StringFormat, StringAlignmentCenter);
  GdipSetStringFormatLineAlign(StringFormat, StringAlignmentCenter);
  GdipCreateSolidFill(ARGBMake($FF, $FF, $FF), Brush);
  
  with TWidgetWindow(Sender).WidgetClientRect do
  begin
    Rect.X := Left;
    Rect.Y := Top;
    Rect.Width := Right - Left;
    Rect.Height := Bottom - Top;
  end;
  
  GdipDrawString(TWidgetWindow(Sender).WidgetDIB.Context, PWCHAR(WideString(Text)),
    Length(Text), Font, @Rect, StringFormat, Brush);

  GdipDeleteBrush(Brush);
  GdipDeleteStringFormat(StringFormat);
  GdipDeleteFont(Font);
  GdipDeleteFontFamily(FontFamily);
end;

procedure TWidgetForm.OnClick(Sender: TObject);
begin
  WidgetApplication.Terminate;
end;

begin
  WidgetForm := TWidgetForm.Create;
  
  WPanel := TWidgetWindow.Create;
  WPanel.WidgetName := 'Panel';
  WPanel.WidgetVisible := True;
  WPanel.WidgetMoveOnMouse := True;
  WPanel.WidgetLeft := 100;
  WPanel.WidgetTop := 100;
  WPanel.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\dwidget.png', 'bckg');
  WPanel.UseRenderAsSize('bckg');
  WidgetApplication.AddWidget(WPanel);

  WButton := TWidgetWindow.Create;
  WButton.WidgetName := 'Button';
  WButton.WidgetVisible := True;
  WButton.WidgetLeft := 10;
  WButton.WidgetTop := 3;
  WButton.OnClick := WidgetForm.OnClick;
  WButton.OnMouseDown := WidgetForm.OnMouseDown;
  WButton.OnMouseUp := WidgetForm.OnMouseDown;
  WButton.OnMouseEnter := WidgetForm.OnMouseEnter;
  WButton.OnMouseLeave := WidgetForm.OnMouseEnter;
  WButton.OnDrawRender := WidgetForm.OnRender;
  WButton.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\startnormal.png', 'normal');
  WButton.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\startover.png', 'over');
  WButton.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\startpressed.png', 'press');
  WButton.UseRenderAsSize('normal');
  WPanel.AddWidget(WButton);

  WLabel := TWidgetWindow.Create;
  WLabel.WidgetName := 'Label';
  WLabel.WidgetVisible := True;
  WLabel.SetWidgetRect2(645, 20, 115, 20);
  WLabel.SetTimer('timer', 500, WidgetForm.OnTimer, False);
  WLabel.OnAfterDrawRender := WidgetForm.OnAfterRender;
  WPanel.AddWidget(WLabel);
  WLabel.SetTimerState('timer', True);

  WClock := TWidgetWindow.Create;
  WClock.WidgetName := 'Clock';
  WClock.WidgetVisible := True;
  WClock.WidgetMoveOnMouse := True;
  WClock.WidgetLeft := 100;
  WClock.WidgetTop := 100;
  WClock.OnMouseEnter := WidgetForm.OnClockMouseEnter;
  WClock.OnMouseLeave := WidgetForm.OnMouseLeave;
  WClock.OnAfterDrawRender := WidgetForm.OnAfterRenderClock;
  WClock.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\clock.png', 'bckg');
  WClock.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\strelka.png', 'arrow');
  WClock.UseRenderAsSize('bckg');
  WidgetApplication.AddWidget(WClock);

  WClose := TWidgetWindow.Create;
  WClose.WidgetName := 'Close';
  WClose.OnMouseLeave := WidgetForm.OnMouseLeave;
  WClose.SetRender(ExtractFilePath(ParamStr(0)) + 'Graphics\closebtn.png', 'bckg');
  WClose.UseRenderAsSize('bckg');
  WClose.WidgetLeft := WClock.WidgetWidth - 5;
  WClose.WidgetTop := 5 - WClose.WidgetHeight;
  WClock.AddWidget(WClose);
  
  WidgetApplication.Run;
  WidgetForm.Free;
end.
